@@ -16,12 +16,9 @@
 #define SOUP_IS_SESSION_ASYNC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), SOUP_TYPE_SESSION_ASYNC))
 #define SOUP_SESSION_ASYNC_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SOUP_TYPE_SESSION_ASYNC, SoupSessionAsyncClass))
 
-typedef struct SoupSessionAsyncPrivate SoupSessionAsyncPrivate;
-
 struct SoupSessionAsync {
 	SoupSession parent;
 
-	SoupSessionAsyncPrivate *priv;
 };
 
 typedef struct {
